@@ -1,18 +1,21 @@
 #include <string.h>
 #include <limits.h>
+#include <syslog.h>
+#include <arpa/inet.h>
 
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 
-#include <openais/service/objdb.h>
-#include <openais/service/config.h>
-#include <openais/lcr/lcr_comp.h>
+#include <corosync/lcr/lcr_comp.h>
+#include <corosync/engine/objdb.h>
+#include <corosync/engine/config.h>
 
 #include "logging.h"
 
 static int xml_readconfig(struct objdb_iface_ver0 *objdb, char **error_string);
+static int xml_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, char **error_string);
 static int init_config(struct objdb_iface_ver0 *objdb, char *configfile, char *error_string);
 static char error_reason[1024];
 static int xmllistindex = 0;
@@ -25,7 +28,8 @@ static char previous_query[PATH_MAX];
  */
 
 static struct config_iface_ver0 xmlconfig_iface_ver0 = {
-	.config_readconfig        = xml_readconfig
+	.config_readconfig        = xml_readconfig,
+	.config_reloadconfig      = xml_reloadconfig
 };
 
 static struct lcr_iface ifaces_ver0[2] = {
@@ -56,7 +60,7 @@ static char *do_xml_query(xmlXPathContextPtr ctx, char *query, int list) {
 	xmlXPathObjectPtr obj = NULL;
 	xmlNodePtr node = NULL;
 	char *rtn = NULL;
-	int size = 0, nnv = 0;
+	int size = 0, nnv = 0, child = 0;
 
 	if (list && !strcmp(query, previous_query))
 		xmllistindex++;
@@ -79,10 +83,11 @@ static char *do_xml_query(xmlXPathContextPtr ctx, char *query, int list) {
 
 		if (((node->type == XML_ATTRIBUTE_NODE) && strstr(query, "@*")) ||
 		     ((node->type == XML_ELEMENT_NODE) && strstr(query, "child::*"))) {
-			if (node->children && node->children->content)
+			if (node->children && node->children->content) {
 				size = strlen((char *)node->children->content) +
 					strlen((char *)node->name)+2;
-			else
+				child = 1;
+			} else
 				size = strlen((char *)node->name)+2;
 
 			nnv = 1;
@@ -97,9 +102,14 @@ static char *do_xml_query(xmlXPathContextPtr ctx, char *query, int list) {
 		if (!rtn)
 			goto fail;
 
-		if (nnv)
-			sprintf(rtn, "%s=%s", node->name, node->children ? (char *)node->children->content:"");
-		else
+		memset(rtn, 0, size);
+
+		if (nnv) {
+			if (child)
+				sprintf(rtn, "%s=%s", node->name, (char *)node->children->content);
+			else
+				sprintf(rtn, "%s=", node->name);
+		} else
 			sprintf(rtn, "%s", node->children ? node->children->content : node->name);
 
 		if(list)
@@ -212,7 +222,7 @@ static int read_config_for(xmlXPathContextPtr ctx, struct objdb_iface_ver0 *objd
 		if (subkeycount > 0 && strcmp(str, subkeys[subkeycount-1]) == 0)
 		{
 			free(str);
-			break;
+			continue;
 		}
 		subkeys[subkeycount++] = str;
 	}
@@ -237,13 +247,19 @@ static int read_config_for(xmlXPathContextPtr ctx, struct objdb_iface_ver0 *objd
 	return gotcount;
 }
 
+static int xml_reloadconfig(struct objdb_iface_ver0 *objdb, int flush, char **error_string)
+{
+	fprintf(stderr, "CC: xml_reloadconfig\n");
+	return xml_readconfig(objdb, error_string);
+}
+
 static int xml_readconfig(struct objdb_iface_ver0 *objdb, char **error_string)
 {
 	int ret = 0;
 	char *configfile = DEFAULT_CONFIG;
 
 	/* We need to set this up to internal defaults too early */
-	openlog("openais", LOG_CONS|LOG_PID, SYSLOGFACILITY);
+	openlog("corosync", LOG_CONS|LOG_PID, SYSLOGFACILITY);
 
 	if(getenv("COROSYNC_CLUSTER_CONFIG_FILE"))
 		configfile = getenv("COROSYNC_CLUSTER_CONFIG_FILE");
@@ -281,15 +297,6 @@ static int init_config(struct objdb_iface_ver0 *objdb, char *configfile, char *e
 	}
 
 	/* load it in objdb */
-
-	/* These first few are just versions of openais.conf */
-	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "totem", "/cluster/totem", 1);
-	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "logging", "/cluster/logging", 1);
-	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "event", "/cluster/event", 1);
-	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "aisexec", "/cluster/aisexec", 1);
-	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "amf", "/cluster/amf", 1);
-
-	/* This is stuff specific to us, eg quorum device timeout */
 	read_config_for(ctx, objdb, OBJECT_PARENT_HANDLE, "cluster", "/cluster", 1);
 
 fail:
