@@ -12,6 +12,7 @@ static struct client *client = NULL;
 static struct pollfd *pollfd = NULL;
 static pthread_t query_thread;
 static pthread_mutex_t query_mutex;
+static struct list_head controlled_entries;
 
 struct client {
 	int fd;
@@ -185,6 +186,15 @@ struct fd *find_fd(char *name)
 	return NULL;
 }
 
+/* We don't require cman dirty/disallowed to detect and handle cpg merges after
+   a partition, because we already do that with started_count checks and our
+   own disallowed flag.  But, we do need cman dirty/disallowed to deal with
+   correctly skipping victims that rejoin the cluster.  Without cman
+   dirty/disallowed, we'd skip fencing a node after a merge of a partition
+   since the merged node would be a cman member and a fenced:daemon cpg member.
+   By setting the dirty flag, cman won't report a dirty merged node as a
+   member, so we'll continue fencing it. */
+
 static int do_join(char *name)
 {
 	struct fd *fd;
@@ -213,6 +223,9 @@ static int do_join(char *name)
 		rv = fd_join_group(fd);
 	else
 		rv = fd_join(fd);
+
+	if (!rv)
+		set_cman_dirty();
  out:
 	return rv;
 }
@@ -538,11 +551,18 @@ void query_unlock(void)
 static void *process_queries(void *arg)
 {
 	struct fenced_header h;
-	int s = *((int *)arg);
-	int f, rv;
+	int f, rv, s;
+
+	rv = setup_listener(FENCED_QUERY_SOCK_PATH);
+	if (rv < 0)
+		exit (-1);
+
+	s = rv;
 
 	for (;;) {
 		f = accept(s, NULL, NULL);
+		if (f < 0)
+			exit (-1);
 
 		rv = do_read(f, &h, sizeof(h));
 		if (rv < 0) {
@@ -584,24 +604,120 @@ static void *process_queries(void *arg)
 
 static int setup_queries(void)
 {
-	int rv, s;
-
-	rv = setup_listener(FENCED_QUERY_SOCK_PATH);
-	if (rv < 0)
-		return rv;
-	s = rv;
+	int rv;
 
 	pthread_mutex_init(&query_mutex, NULL);
 
-	rv = pthread_create(&query_thread, NULL, process_queries, &s);
+	rv = pthread_create(&query_thread, NULL, process_queries, NULL);
 	if (rv < 0) {
 		log_error("can't create query thread");
-		close(s);
 		return rv;
 	}
 	return 0;
 }
 
+struct controlled_entry {
+	struct list_head list;
+	char path[PATH_MAX+1];
+};
+
+static void register_controlled_dir(char *path)
+{
+	struct controlled_entry *ce;
+
+	ce = malloc(sizeof(struct controlled_entry));
+	if (!ce)
+		return;
+	memset(ce, 0, sizeof(struct controlled_entry));
+	strncpy(ce->path, path, PATH_MAX);
+	list_add(&ce->list, &controlled_entries);
+}
+
+static int ignore_nolock(char *sysfs_dir, char *table)
+{
+	char path[PATH_MAX];
+	int fd;
+
+	memset(path, 0, PATH_MAX);
+
+	snprintf(path, PATH_MAX, "%s/%s/lock_module/proto_name",
+		 sysfs_dir, table);
+
+	/* lock_nolock doesn't create the "lock_module" dir at all,
+	   so we'll fail to open this */
+
+	fd = open(path, O_RDONLY);
+	if (fd < 0)
+		return 1;
+
+	close(fd);
+	return 0;
+}
+
+static int check_controlled_dir(char *path)
+{
+	DIR *d;
+	struct dirent *de;
+	int count = 0;
+
+	d = opendir(path);
+	if (!d)
+		return 0;
+
+	while ((de = readdir(d))) {
+		if (de->d_name[0] == '.')
+			continue;
+
+		if (strstr(path, "fs/gfs") && ignore_nolock(path, de->d_name))
+			continue;
+
+		log_error("found uncontrolled entry %s/%s", path, de->d_name);
+		count++;
+	}
+	closedir(d);
+
+	return count;
+}
+
+/* Joining the "fenced:daemon" cpg (in setup_cpg()) tells fenced on other
+   nodes that we are in a "clean state", and don't need fencing.  So, if
+   we're a pending fence victim on another node, they'll skip fencing us
+   once we start fenced and join the "daemon" cpg (it's not the fence domain
+   cpg which we join when fence_tool join is run).  This "daemon" cpg is just
+   to notify others that we have no uncontrolled gfs/dlm objects.
+   (Conceptually, we could use the fence domain cpg for this purpose instead,
+   but that would require processing domain membership changes during
+   fence_victims(), which would be a major change in the way the daemon works.)
+
+   So, if we (the local node) are *not* in a clean state, we don't join the
+   daemon cpg and we exit; we still need to be fenced.  If we are starting
+   up and find that instances of gfs/dlm in the kernel have been previously
+   abandoned, that's an unclean, unreset state, and we still need fencing. */
+
+static int check_uncontrolled_entries(void)
+{
+	struct controlled_entry *ce;
+	int count = 0;
+
+	list_for_each_entry(ce, &controlled_entries, list) {
+		if (strncmp(ce->path, "-", 1))
+			goto skip_default;
+	}
+
+	/* the default dirs to check */
+	register_controlled_dir("/sys/kernel/dlm");
+	register_controlled_dir("/sys/fs/gfs2");
+	register_controlled_dir("/sys/fs/gfs");
+
+ skip_default:
+	list_for_each_entry(ce, &controlled_entries, list)
+		count += check_controlled_dir(ce->path);
+
+	if (count)
+		return -1;
+	return 0;
+}
+
 void cluster_dead(int ci)
 {
 	log_error("cluster is down, exiting");
@@ -634,6 +750,15 @@ static void loop(void)
 
 	setup_logging();
 
+	rv = check_uncontrolled_entries();
+	if (rv < 0)
+		goto out;
+
+	rv = setup_cpg();
+	if (rv < 0)
+		goto out;
+	client_add(rv, process_cpg, cluster_dead);
+
 	group_mode = GROUP_LIBCPG;
 
 	if (cfgd_groupd_compat) {
@@ -648,15 +773,6 @@ static void loop(void)
 	}
 	log_debug("group_mode %d compat %d", group_mode, cfgd_groupd_compat);
 
-	if (group_mode == GROUP_LIBCPG) {
-		/*
-		rv = setup_cpg();
-		if (rv < 0)
-			goto out;
-		client_add(rv, process_cpg, cluster_dead);
-		*/
-	}
-
 	for (;;) {
 		rv = poll(pollfd, client_maxi + 1, -1);
 		if (rv == -1 && errno == EINTR) {
@@ -692,6 +808,7 @@ static void loop(void)
  out:
 	if (cfgd_groupd_compat)
 		close_groupd();
+	close_cpg();
 	close_logging();
 	close_ccs();
 	close_cman();
@@ -757,7 +874,10 @@ static void print_usage(void)
 	printf("               1: use libgroup for compat with cluster2/rhel5\n");
 	printf("               2: use groupd to detect old, or mode 1, nodes that\n"
 	       "               require compat, use libcpg if none found\n");
-	printf("  -c           All nodes are in a clean state to start\n");
+	printf("  -r <path>    Register a directory that needs to be empty for\n");
+	printf("               the daemon to start.  \"-\" to skip default directories\n");
+	printf("               /sys/fs/gfs, /sys/fs/gfs2, /sys/kernel/dlm\n");
+	printf("  -c           All nodes are in a clean state to start; do no startup fencing\n");
 	printf("  -s           Skip startup fencing of nodes with no defined fence methods\n");
 	printf("  -j <secs>    Post-join fencing delay (default %d)\n", DEFAULT_POST_JOIN_DELAY);
 	printf("  -f <secs>    Post-fail fencing delay (default %d)\n", DEFAULT_POST_FAIL_DELAY);
@@ -772,7 +892,7 @@ static void print_usage(void)
 	printf("\n");
 }
 
-#define OPTION_STRING	"L:g:cj:f:Dn:O:hVSs"
+#define OPTION_STRING	"L:g:cj:f:Dn:O:hVSse:r:"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -830,6 +950,10 @@ static void read_arguments(int argc, char **argv)
 			cfgd_override_path = strdup(optarg);
 			break;
 
+		case 'r':
+			register_controlled_dir(optarg);
+			break;
+
 		case 'h':
 			print_usage();
 			exit(EXIT_SUCCESS);
@@ -879,6 +1003,7 @@ static void set_oom_adj(int val)
 int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&domains);
+	INIT_LIST_HEAD(&controlled_entries);
 
 	init_logging();
 
