@@ -494,6 +494,10 @@ int lock_dlm_leave(struct mount_options *mo, struct gen_sb *sb, int mnterr)
 
 	strncpy(ma.dir, mo->dir, PATH_MAX);
 	strncpy(ma.type, fsname, PATH_MAX);
+	if (mo->locktable[0])
+		strncpy(ma.table, mo->locktable, PATH_MAX);
+	else
+		strncpy(ma.table, sb->locktable, PATH_MAX);
 
 	rv = gfsc_fs_leave(&ma, mnterr);
 	if (rv)
@@ -510,8 +514,10 @@ int lock_dlm_remount(struct mount_options *mo, struct gen_sb *sb)
 
 	memset(&ma, 0, sizeof(ma));
 
-	/* FIXME: how to check for spectator remounts, we want
-	   to disallow remount to/from spectator */
+	if (strstr(mo->extra, "spectator")) {
+		warn("spectator remounts not allowed");
+		return -1;
+	}
 
 	if (mo->flags & MS_RDONLY)
 		mode = "ro";
@@ -521,6 +527,11 @@ int lock_dlm_remount(struct mount_options *mo, struct gen_sb *sb)
 	strncpy(ma.dir, mo->dir, PATH_MAX);
 	strncpy(ma.type, fsname, PATH_MAX);
 	strncpy(ma.options, mode, PATH_MAX);
+	if (mo->locktable[0])
+		strncpy(ma.table, mo->locktable, PATH_MAX);
+	else
+		strncpy(ma.table, sb->locktable, PATH_MAX);
+
 
 	fd = gfsc_fs_connect();
 	if (fd < 0) {
