@@ -1,6 +1,5 @@
 #include "dlm_daemon.h"
 #include "config.h"
-#include "libfenced.h"
 
 uint32_t cpgname_to_crc(const char *data, int len);
 
@@ -386,9 +385,8 @@ static void node_history_fail(struct lockspace *ls, int nodeid,
 static int check_fencing_done(struct lockspace *ls)
 {
 	struct node *node;
-	struct fenced_node nodeinfo;
-	struct fenced_domain domain;
-	int wait_count = 0;
+	uint64_t last_fenced_time;
+	int in_progress, wait_count = 0;
 	int rv;
 
 	if (!cfgd_enable_fencing)
@@ -401,13 +399,11 @@ static int check_fencing_done(struct lockspace *ls)
 		/* check with fenced to see if the node has been
 		   fenced since node->add_time */
 
-		memset(&nodeinfo, 0, sizeof(nodeinfo));
-
-		rv = fenced_node_info(node->nodeid, &nodeinfo);
+		rv = fence_node_time(node->nodeid, &last_fenced_time);
 		if (rv < 0)
 			log_error("fenced_node_info error %d", rv);
 
-		if (nodeinfo.last_fenced_time > node->add_time) {
+		if (last_fenced_time > node->add_time) {
 			node->check_fencing = 0;
 			node->add_time = 0;
 		} else {
@@ -424,13 +420,13 @@ static int check_fencing_done(struct lockspace *ls)
 	   we may not have seen in any lockspace), and return 0 if there
 	   are any */
 
-	rv = fenced_domain_info(&domain);
+	rv = fence_in_progress(&in_progress);
 	if (rv < 0) {
 		log_error("fenced_domain_info error %d", rv);
 		return 0;
 	}
 
-	if (domain.victim_count)
+	if (in_progress)
 		return 0;
 	return 1;
 }
@@ -443,18 +439,18 @@ static int check_quorum_done(struct lockspace *ls)
 	if (!cfgd_enable_quorum)
 		return 1;
 
-	/* wait for cman to see all the same nodes failed, so we know that
-	   cman_quorate is adjusted for the same failures we've seen
-	   (see comment in fenced about the assumption here) */
+	/* wait for quorum system (cman) to see all the same nodes failed, so
+	   we know that cluster_quorate is adjusted for the same failures we've
+	   seen (see comment in fenced about the assumption here) */
 
 	list_for_each_entry(node, &ls->node_history, list) {
 		if (!node->check_quorum)
 			continue;
 
-		if (!is_cman_member(node->nodeid)) {
+		if (!is_cluster_member(node->nodeid)) {
 			node->check_quorum = 0;
 		} else {
-			log_group(ls, "check_quorum %d is_cman_member",
+			log_group(ls, "check_quorum nodeid %d is_cluster_member",
 				  node->nodeid);
 			wait_count++;
 		}
@@ -463,7 +459,7 @@ static int check_quorum_done(struct lockspace *ls)
 	if (wait_count)
 		return 0;
 
-	if (!cman_quorate) {
+	if (!cluster_quorate) {
 		log_group(ls, "check_quorum not quorate");
 		return 0;
 	}
@@ -490,7 +486,7 @@ static int check_fs_done(struct lockspace *ls)
 		if (node->fs_notified) {
 			node->check_fs = 0;
 		} else {
-			log_group(ls, "check_fs %d needs fs notify",
+			log_group(ls, "check_fs nodeid %d needs fs notify",
 				  node->nodeid);
 			wait_count++;
 		}
@@ -553,11 +549,11 @@ static void start_kernel(struct lockspace *ls)
 	struct change *cg = list_first_entry(&ls->changes, struct change, list);
 
 	if (!ls->kernel_stopped) {
-		log_error("start_kernel %u not stopped", cg->seq);
+		log_error("start_kernel cg %u not stopped", cg->seq);
 		return;
 	}
 
-	log_group(ls, "start_kernel %u member_count %d",
+	log_group(ls, "start_kernel cg %u member_count %d",
 		  cg->seq, cg->member_count);
 
 	/* needs to happen before setting control which starts recovery */
@@ -580,7 +576,7 @@ static void start_kernel(struct lockspace *ls)
 static void stop_kernel(struct lockspace *ls, uint32_t seq)
 {
 	if (!ls->kernel_stopped) {
-		log_group(ls, "stop_kernel %u", seq);
+		log_group(ls, "stop_kernel cg %u", seq);
 		set_sysfs_control(ls->name, 0);
 		ls->kernel_stopped = 1;
 	}
@@ -634,11 +630,11 @@ static int wait_messages_done(struct lockspace *ls)
 	}
 
 	if (need) {
-		log_group(ls, "wait_messages_done need %d of %d", need, total);
+		log_group(ls, "wait_messages need %d of %d", need, total);
 		return 0;
 	}
 
-	log_group(ls, "wait_messages_done got all %d", total);
+	log_group(ls, "wait_messages got all %d", total);
 	return 1;
 }
 
@@ -746,15 +742,15 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	id = get_id_struct(ids, li->id_info_count, li->id_info_size,our_nodeid);
 
 	if (!id) {
-		log_debug("match_change fail %d:%u we are not in members",
-			  hd->nodeid, seq);
+		log_group(ls, "match_change %d:%u skip %u we are not in members",
+			  hd->nodeid, seq, cg->seq);
                 return 0;
 	}
 
 	memb = find_memb(cg, hd->nodeid);
 	if (!memb) {
-		log_group(ls, "match_change fail %d:%u sender not member",
-			  hd->nodeid, seq);
+		log_group(ls, "match_change %d:%u skip %u sender not member",
+			  hd->nodeid, seq, cg->seq);
 		return 0;
 	}
 
@@ -765,8 +761,8 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	    li->joined_count != cg->joined_count ||
 	    li->remove_count != cg->remove_count ||
 	    li->failed_count != cg->failed_count) {
-		log_group(ls, "match_change fail %d:%u expect counts "
-			  "%d %d %d %d", hd->nodeid, seq,
+		log_group(ls, "match_change %d:%u skip %u expect counts "
+			  "%d %d %d %d", hd->nodeid, seq, cg->seq,
 			  cg->member_count, cg->joined_count,
 			  cg->remove_count, cg->failed_count);
 		return 0;
@@ -778,8 +774,8 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	for (i = 0; i < li->id_info_count; i++) {
 		memb = find_memb(cg, id->nodeid);
 		if (!memb) {
-			log_group(ls, "match_change fail %d:%u no memb %d",
-			  	  hd->nodeid, seq, id->nodeid);
+			log_group(ls, "match_change %d:%u skip %u no memb %d",
+			  	  hd->nodeid, seq, cg->seq, id->nodeid);
 			members_mismatch = 1;
 			break;
 		}
@@ -788,7 +784,8 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	if (members_mismatch)
 		return 0;
 
-	log_group(ls, "match_change done %d:%u", hd->nodeid, seq);
+	log_group(ls, "match_change %d:%u matches cg %u", hd->nodeid, seq,
+		  cg->seq);
 	return 1;
 }
 
@@ -875,13 +872,9 @@ static void receive_start(struct lockspace *ls, struct dlm_header *hd, int len)
 		log_error("receive_start %d:%u add node with started_count %u",
 			  hd->nodeid, seq, li->started_count);
 
-		/* observe this scheme working before using it; I'm not sure
-		   that a joining node won't ever see an existing node as added
-		   under normal circumstances */
-		/*
+		/* see comment in fence/fenced/cpg.c */
 		memb->disallowed = 1;
 		return;
-		*/
 	}
 
 	node_history_start(ls, hd->nodeid);
@@ -982,7 +975,7 @@ static void send_info(struct lockspace *ls, int type)
 		id++;
 	}
 
-	log_group(ls, "send_%s %u flags %x counts %u %d %d %d %d",
+	log_group(ls, "send_%s cg %u flags %x counts %u %d %d %d %d",
 		  type == DLM_MSG_START ? "start" : "plocks_stored",
 		  cg->seq, hd->flags, ls->started_count, cg->member_count,
 		  cg->joined_count, cg->remove_count, cg->failed_count);
@@ -1165,8 +1158,11 @@ static int add_change(struct lockspace *ls,
 		else
 			node_history_left(ls, memb->nodeid, cg);
 
-		log_group(ls, "add_change %u nodeid %d remove reason %d",
+		log_group(ls, "add_change cg %u remove nodeid %d reason %d",
 			  cg->seq, memb->nodeid, left_list[i].reason);
+
+		if (left_list[i].reason == CPG_REASON_PROCDOWN)
+			kick_node_from_cluster(memb->nodeid);
 	}
 
 	for (i = 0; i < joined_list_entries; i++) {
@@ -1183,19 +1179,19 @@ static int add_change(struct lockspace *ls,
 		else
 			node_history_init(ls, memb->nodeid, cg);
 
-		log_group(ls, "add_change %u nodeid %d joined", cg->seq,
+		log_group(ls, "add_change cg %u joined nodeid %d", cg->seq,
 			  memb->nodeid);
 	}
 
 	if (cg->we_joined) {
-		log_group(ls, "add_change %u we joined", cg->seq);
+		log_group(ls, "add_change cg %u we joined", cg->seq);
 		list_for_each_entry(memb, &cg->members, list)
 			node_history_init(ls, memb->nodeid, cg);
 	}
 
-	log_group(ls, "add_change %u member %d joined %d remove %d failed %d",
-		  cg->seq, cg->member_count, cg->joined_count, cg->remove_count,
-		  cg->failed_count);
+	log_group(ls, "add_change cg %u counts member %d joined %d remove %d "
+		  "failed %d", cg->seq, cg->member_count, cg->joined_count,
+		  cg->remove_count, cg->failed_count);
 
 	list_add(&cg->list, &ls->changes);
 	*cg_out = cg;
@@ -1522,13 +1518,17 @@ int set_fs_notified(struct lockspace *ls, int nodeid)
 
 	/* this shouldn't happen */
 	node = get_node_history(ls, nodeid);
-	if (!node)
+	if (!node) {
+		log_error("set_fs_notified no nodeid %d", nodeid);
 		return -ESRCH;
+	}
 
 	/* this can happen, we haven't seen a nodedown for this node yet,
 	   but we should soon */
-	if (!node->check_fs)
+	if (!node->check_fs) {
+		log_group(ls, "set_fs_notified %d zero check_fs", nodeid);
 		return -EAGAIN;
+	}
 
 	node->fs_notified = 1;
 	return 0;
