@@ -22,7 +22,7 @@ struct client {
 	struct mountgroup *mg;
 };
 
-static void do_leave(char *table, int mnterr);
+static void do_withdraw(char *table);
 
 int do_read(int fd, void *buf, size_t count)
 {
@@ -275,6 +275,12 @@ static void process_uevent(int ci)
 	if (!strstr(buf, "gfs"))
 		return;
 
+	/* if an fs is named "gfs", it results in dlm uevents
+	   like "remove@/kernel/dlm/gfs" */
+
+	if (strstr(buf, "kernel/dlm"))
+		return;
+
 	log_debug("uevent: %s", buf);
 
 	if (strstr(buf, "lock_module"))
@@ -296,7 +302,10 @@ static void process_uevent(int ci)
 		if (lock_module)
 			return;
 
-		do_leave(argv[3], 0);
+		if (group_mode == GROUP_LIBGROUP)
+			do_leave_old(argv[3], 0);
+		else
+			do_leave(argv[3], 0);
 
 	} else if (!strcmp(act, "change@")) {
 		if (!lock_module)
@@ -311,10 +320,7 @@ static void process_uevent(int ci)
 		if (!lock_module)
 			return;
 
-		if (group_mode == GROUP_LIBGROUP)
-			do_withdraw_old(argv[3]);
-		else
-			log_error("TODO withdraw for libcpg");
+		do_withdraw(argv[3]);
 
 	} else {
 		if (!lock_module)
@@ -454,6 +460,9 @@ static void query_mountgroup_info(int fd, char *name)
 		goto out;
 	}
 
+	memset(&mountgroup, 0, sizeof(mountgroup));
+	mountgroup.group_mode = group_mode;
+
 	if (group_mode == GROUP_LIBGROUP)
 		rv = set_mountgroup_info_group(mg, &mountgroup);
 	else
@@ -558,19 +567,6 @@ static void query_mountgroup_nodes(int fd, char *name, int option, int max)
 		free(nodes);
 }
 
-void client_reply_remount(struct mountgroup *mg, int result)
-{
-	struct gfsc_mount_args *ma = &mg->mount_args;
-
-	log_group(mg, "client_reply_remount ci %d result %d",
-		  mg->remount_client, result);
-
-	do_reply(client[mg->remount_client].fd, GFSC_CMD_FS_REMOUNT,
-		 mg->name, result, ma, sizeof(struct gfsc_mount_args));
-
-	mg->remount_client = 0;
-}
-
 void client_reply_join(int ci, struct gfsc_mount_args *ma, int result)
 {
 	char *name = strstr(ma->table, ":") + 1;
@@ -604,7 +600,7 @@ void client_reply_join_full(struct mountgroup *mg, int result)
 	if (nodir_str[0])
 		strcat(mg->mount_args.hostdata, nodir_str);
  out:
-	log_group(mg, "client_reply_join_full ci %d result %d hostdata %s",
+	log_group(mg, "client_reply_join_full ci %d result %d %s",
 		  mg->mount_client, result, mg->mount_args.hostdata);
 
 	client_reply_join(mg->mount_client, &mg->mount_args, result);
@@ -734,16 +730,42 @@ static void do_join(int ci, struct gfsc_mount_args *ma)
 	client_reply_join(ci, ma, rv);
 }
 
-static void do_leave(char *table, int mnterr)
+/* The basic rule of withdraw is that we don't want to tell the kernel to drop
+   all locks until we know gfs has been stopped/blocked on all nodes.
+   A withdrawing node is very much like a readonly node, differences are
+   that others recover its journal when they remove it from the group,
+   and when it's been removed from the group, it tells the locally withdrawing
+   gfs to clear out locks. */
+
+static void do_withdraw(char *table)
 {
+	struct mountgroup *mg;
 	char *name = strstr(table, ":") + 1;
+	int rv;
 
-	log_debug("leave: %s mnterr %d", name, mnterr);
+	log_debug("withdraw: %s", name);
 
-	if (group_mode == GROUP_LIBGROUP)
-		gfs_leave_mountgroup_old(name, mnterr);
-	else
-		gfs_leave_mountgroup(name, mnterr);
+	if (!cfgd_enable_withdraw) {
+		log_error("withdraw feature not enabled");
+		return;
+	}
+
+	mg = find_mg(name);
+	if (!mg) {
+		log_error("do_withdraw no mountgroup %s", name);
+		return;
+	}
+
+	mg->withdraw_uevent = 1;
+
+	rv = run_dmsetup_suspend(mg, mg->mount_args.dev);
+	if (rv) {
+		log_error("do_withdraw %s: dmsetup %s error %d", mg->name,
+			  mg->mount_args.dev, rv);
+		return;
+	}
+
+	dmsetup_wait = 1;
 }
 
 static void do_mount_done(char *table, int result)
@@ -769,6 +791,53 @@ static void do_mount_done(char *table, int result)
 		gfs_mount_done(mg);
 }
 
+void client_reply_remount(struct mountgroup *mg, int ci, int result)
+{
+	do_reply(client[ci].fd, GFSC_CMD_FS_REMOUNT, mg->name, result,
+		 &mg->mount_args, sizeof(struct gfsc_mount_args));
+}
+
+/* mount.gfs creates a special ma->options string with only "ro" or "rw" */
+
+static void do_remount(int ci, struct gfsc_mount_args *ma)
+{
+	struct mountgroup *mg;
+	char *name = strstr(ma->table, ":") + 1;
+	int ro = 0, result = 0;
+
+	log_debug("remount: %s ci %d options %s", name, ci, ma->options);
+
+	mg = find_mg(name);
+	if (!mg) {
+		log_error("remount: %s not found", name);
+		result = -1;
+		goto out;
+	}
+
+	if (mg->spectator) {
+		log_error("remount of spectator not allowed");
+		result = -1;
+		goto out;
+	}
+
+	if (!strcmp(ma->options, "ro"))
+		ro = 1;
+
+	if ((mg->ro && ro) || (!mg->ro && !ro))
+		goto out;
+
+	if (group_mode == GROUP_LIBGROUP) {
+		/* the receive calls client_reply_remount */
+		mg->remount_client = ci;
+		send_remount_old(mg, ma);
+		return;
+	}
+
+	send_remount(mg, ma);
+ out:
+	client_reply_remount(mg, ci, result);
+}
+
 void process_connection(int ci)
 {
 	struct gfsc_header h;
@@ -830,7 +899,10 @@ void process_connection(int ci)
 		break;
 
 	case GFSC_CMD_FS_LEAVE:
-		do_leave(ma->table, h.data);
+		if (group_mode == GROUP_LIBGROUP)
+			do_leave_old(ma->table, h.data);
+		else
+			do_leave(ma->table, h.data);
 		break;
 
 	case GFSC_CMD_FS_MOUNT_DONE:
@@ -838,13 +910,7 @@ void process_connection(int ci)
 		break;
 
 	case GFSC_CMD_FS_REMOUNT:
-		if (group_mode == GROUP_LIBGROUP)
-			remount_mountgroup_old(ci, ma);
-#if 0
-		/* FIXME */
-		else
-			remount_mountgroup(ci, ma);
-#endif
+		do_remount(ci, ma);
 		break;
 
 	default:
@@ -926,11 +992,18 @@ void query_unlock(void)
 static void *process_queries(void *arg)
 {
 	struct gfsc_header h;
-	int s = *((int *)arg);
-	int f, rv;
+	int f, rv, s;
+
+	rv = setup_listener(GFSC_QUERY_SOCK_PATH);
+	if (rv < 0)
+		exit (-1);
+
+	s = rv;
 
 	for (;;) {
 		f = accept(s, NULL, NULL);
+		if (f < 0)
+			exit (-1);
 
 		rv = do_read(f, &h, sizeof(h));
 		if (rv < 0) {
@@ -978,19 +1051,13 @@ static void *process_queries(void *arg)
 
 static int setup_queries(void)
 {
-	int rv, s;
-
-	rv = setup_listener(GFSC_QUERY_SOCK_PATH);
-	if (rv < 0)
-		return rv;
-	s = rv;
+	int rv;
 
 	pthread_mutex_init(&query_mutex, NULL);
 
-	rv = pthread_create(&query_thread, NULL, process_queries, &s);
+	rv = pthread_create(&query_thread, NULL, process_queries, NULL);
 	if (rv < 0) {
 		log_error("can't create query thread");
-		close(s);
 		return rv;
 	}
 	return 0;
@@ -1016,9 +1083,6 @@ static void loop(void)
 	void (*workfn) (int ci);
 	void (*deadfn) (int ci);
 
-	/* FIXME: add code that looks for uncontrolled instances of
-	   gfs filesystems in the kernel */
-
 	rv = setup_queries();
 	if (rv < 0)
 		goto out;
@@ -1039,6 +1103,10 @@ static void loop(void)
 
 	setup_logging();
 
+	rv = check_uncontrolled_filesystems();
+	if (rv < 0)
+		goto out;
+
 	rv = setup_uevent();
 	if (rv < 0)
 		goto out;
@@ -1066,6 +1134,11 @@ static void loop(void)
 		 */
 
 		rv = setup_cpg();
+		if (rv < 0)
+			goto out;
+		client_add(rv, process_cpg, cluster_dead);
+
+		rv = set_protocol();
 		if (rv < 0)
 			goto out;
 
@@ -1156,6 +1229,10 @@ static void loop(void)
 		query_unlock();
 	}
  out:
+	if (group_mode == GROUP_LIBCPG)
+		close_cpg();
+	else if (group_mode == GROUP_LIBGROUP)
+		close_cpg_old();
 	if (cfgd_groupd_compat)
 		close_groupd();
 	close_logging();
@@ -1378,6 +1455,7 @@ static void set_scheduler(void)
 int main(int argc, char **argv)
 {
 	INIT_LIST_HEAD(&mountgroups);
+	INIT_LIST_HEAD(&withdrawn_mounts);
 
 	init_logging();
 
@@ -1434,7 +1512,8 @@ int dump_wrap;
 char plock_dump_buf[GFSC_DUMP_SIZE];
 int plock_dump_len;
 int dmsetup_wait;
-cpg_handle_t libcpg_handle;
+cpg_handle_t cpg_handle_daemon;
 int libcpg_flow_control_on;
 int group_mode;
+struct list_head withdrawn_mounts;
 
